package Flexstor;
use strict;
use vars qw($VERSION);

use Carp;
use SGL;
use TABLE;


$VERSION = '1.0';
BEGIN {
    $Flexstor::file = "Null";
}
1;

sub VERSION {
    # Version of Flexstor
    return $Flexstor::VERSION;
}

sub new {
    croak("Usage: new Flexstor( file )")
      unless(@_ == 2);

    my $that  = shift;
    my $class = ref($that) || $that;
    my $file  = shift;
    if (!-e $file) { 
	warn("File $file not found");
	return -1;
    }
    my @content = read_Flexstor_file($file);
    my $self  = {
        CONTENT => \@content,
	FILE    => $file,
    };
    bless $self, $class;
    return $self;
}

sub read_Flexstor_file {
    my $file = shift;
    my (@items) = split("/",$file);
    my $filepath="";
    my $i;
    for ($i=0;$i<$#items;$i++) {
	$filepath = $filepath.$items[$i]."/";
    }
    open(FP,$file);
    my @content = <FP>;
    close(FP);
    my @lines = grep /include/, @content;
    for ($i=0;$i<=$#lines;$i++) {
	my $line = $lines[$i];
	$line =~ s/^[ ]+//;
	my ($gabba,$file) = split(" ",$line);
	$file =~ s/\"//g;
	$file = $filepath.$file;
	open(FP,$file) || warn("Can not open include file $file");
	my @inc_content = <FP>;
	close(FP);
	@content = (@content,@inc_content);
    }
    return @content;
}

sub sgl
{
    my $self = shift;
    my $sgl_symbol = shift;
    my @lines = ();
    @lines = grep /$sgl_symbol/, @{$self->{CONTENT}};
    if ($#lines<0) {
	warn("Single line symbol $sgl_symbol not found");
    }
    my $i;
    my @sgl = ();
    for ($i=0;$i<=$#lines;$i++) {
	my $line = $lines[$i];
	$line =~ s/\n//;
	$sgl[$i] = new SGL($line);
    }
    return @sgl;
}

sub sgl_value {
    my $self = shift;
    my $sgl_symbol = shift;
    my @lines = ();
    @lines = grep /$sgl_symbol/, @{$self->{CONTENT}};
    if ($#lines<0) {
	warn("Single line symbol $sgl_symbol not found");
    }
    my $i;
    my $sgl;
    my $line = $lines[0];
    $line =~ s/\n//;
    $sgl = new SGL($line);
    return $sgl->value();
}

sub table
{
    my $self = shift;
    my $table_identifier = shift;
    my @content = @{$self->{CONTENT}};
    my $no_tables = grep /$table_identifier/, @content;
    if ($no_tables<0) {
	warn("No tables with identifier $table_identifier found");
    }
    my $i;
    my $j;
    my $i_table;
    my @tables = ();
    $no_tables = $no_tables/2; #There os both begin and end so divide
                               # by 2 to get number of tables.
    for ($i=0;$i<=$#content;$i++) {
	my $line = $content[$i];
	$line =~ s/\n//;
	if ( grep /$table_identifier/, $line ) {
	    # Found begin table
	    my $i_begin = $i;
	    my $i_end = $i;
	    # Get end table
	    for ($j=$i+1;$j<=$#content;$j++) {
		my $line = $content[$j];
		if ( grep /$table_identifier/, $line ) { 
		    $i_end = $j;
		    last;
		}
	    }
	    $tables[$i_table++] = new TABLE(@content[$i_begin..$i_end]);
	    $i = $i_end+1;   # Advance $i beyond this table
	} 
    }
    return @tables;
}

sub get_sgl {
    my $file      = shift;
    my $sgl_symbol= shift;
    my $f         = new Flexstor($file);
    my @sgl       = $f->sgl($sgl_symbol);
    my @sgl_value = $sgl[0]->data();
    return $sgl_value[0];
}


__END__

=head1 NAME

Flexstor - Query and/or read Flexstor files

=head1 SYNOPSIS

DOCUMENTATION NOT YET FINISHED...................

  	use Flexstor;
        # The rest is too complicated for a synopsis; keep reading.

=head1 DESCRIPTION

$f = new Flexstor($file);  # Open and reads Flexstor file $file.

# Single line information

$sgl = $f->sgl($single_line_symbol); # $sgl is the number of single
                           # lines of type $single_line_symbol
                             
@sgl = $f->sgl($single_line_symbol); # @sgl is content of all lines
                           # of type $single_line_symbol. Appears in 
                           # the order which they are encountered in 
                           # the file. 


=head1 BUGS

Included files are not included in their original position when
the Flexstor file is read.

=head1 AUTHOR

Flexstor was written by Arve Kylling.

=cut

