#
# Copyright (C) 1997 Arve Kylling
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To obtain a copy of the GNU General Public License write to the
# Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
# USA.

#-------------------------------------------------------------------------

use lib "../tools";
use UVTools;
use Getopt::Long;

$answers      = 0;

$ret = GetOptions(
		  "--answers",\$answers);
if ($ret==0) {
    exit;
}

my $failure=0;
print "Running various libRadtran tests. This may take some time....\n";

print "First some uvspec tests\n";
$failure+=uvspec_test();
print "Now a Calc_J test\n";
$failure+=calc_j_test();
#$failure+=snowalbedo_test(); FIXME NOT YET IMPLEMENTED
if ( $answers == 1 ) {
	printf stderr "All *.OUT files replaced.\n"
}
else {
    if ($failure) {
	printf stderr "$failure of the tests failed.\n"
	}
    else {
	printf stderr "All tests succeeded.\n"
	}
}
exit;

sub snowalbedo_test {
    my $snowalbedo_failure=0;

    $test = "snowalbedo test 1"; 
    my $inp = "TMP.INP";
    my $out = "TMP.OUT";
    my $exinp  = "../examples/UVSPEC_CLEAR.INP";
    my $exout  = "../examples/UVSPEC_CLEAR.OUT";
    $snowalbedo_failure+=snowalbedo_run_test($inp, $out, $exinp, $exout, $test);

    if ( $answers != 1 ) {
	if ($snowalbedo_failure) {
	    printf stderr "$snowalbedo_failure of the snowalbedo tests failed.\n";
	}
	else {
	    printf stderr "All snowalbedo tests succeeded.\n";
	}
    }
    else {
	printf stderr "All *.OUT files replaced.\n";
    }
    $snowalbedo_failure;
}
   
sub calc_j_test {
    my $calc_j_failure=0;

    $test = "j value"; 
    my $inp = "TMP.INP";
    my $out = "TMP.OUT";
    my $exinp  = "../examples/J_UVSPEC.INP";
    my $exout  = "../examples/J.OUT";
    $calc_j_failure+=calc_j_run_test($inp, $out, $exinp, $exout, $test);

    if ( $answers != 1 ) {
	if ($calc_j_failure) {
	    printf stderr "$calc_j_failure of the calc_j tests failed.\n";
	}
	else {
	    printf stderr "All calc_j tests succeeded.\n";
	}
    }
    else {
	printf stderr "All *.OUT files replaced.\n";
    }
    $calc_j_failure;
}
   
sub calc_j_run_test {

    my ($inp, $out, $exinp, $exout, $test) = @_;
    my $diffout= "TMP_DIFF.OUT";
    system("cat  $exinp > $inp");
    my $uvspec_out = "../examples/J_UVSPEC.OUT";
    system("$UVTools::uvspec < $inp > $uvspec_out");
    my $temp = 300.0;
    my $string = "'self_surface_temperature(K) $temp'";
    system("echo $string >> $uvspec_out");
    system("perl ../tools/Calc_J.pl $uvspec_out > $out");
    if ( $answers == 1 ) {
	system("mv $out $exout");
	system("rm $inp");
    }
    else {
	system("perl ../tools/ndiff --limit 0.01 --maxdiff 0.005 $exout $out > $diffout");
	open(FILE,$diffout);
	$ndiffs = <FILE>;
	close(FILE);
	chop $ndiffs;
	printf stderr "Found %d serious differences for $test test.\n";
	system("rm $diffout $inp $out");
	return $ndiffs;
    }
}
sub uvspec_test {
    my $uvspec_failure=0;

    $test = "uvspec clear sky"; 
    my $inp = "TMP.INP";
    my $out = "TMP.OUT";
    my $exinp  = "../examples/UVSPEC_CLEAR.INP";
    my $exout  = "../examples/UVSPEC_CLEAR.OUT";
    $uvspec_failure+=uvspec_run_test($inp, $out, $exinp, $exout, $test);

    $test = "uvspec aerosol"; 
    $inp = "TMP.INP";
    $out = "TMP.OUT";
    $exinp  = "../examples/UVSPEC_CLEAR.INP ../examples/UVSPEC_AEROSOL.INP";
    $exout  = "../examples/UVSPEC_AEROSOL.OUT";
    $uvspec_failure+=uvspec_run_test($inp, $out, $exinp, $exout, $test);
    
    $test = "uvspec radiances"; 
    $inp = "TMP.INP";
    $out = "TMP.OUT";
    $exinp  = "../examples/UVSPEC_CLEAR.INP ../examples/UVSPEC_RADIANCES.INP";
    $exout  = "../examples/UVSPEC_RADIANCES.OUT";
    $uvspec_failure+=uvspec_run_test($inp, $out, $exinp, $exout, $test);

    if ( $answers != 1 ) {
	if ($uvspec_failure) {
	    printf stderr "$uvspec_failure of the uvspec tests failed.\n";
	}
	else {
	    printf stderr "All uvspec tests succeeded.\n";
	}
    }
    else {
	printf stderr "All *.OUT files replaced.\n";
    }
    $uvspec_failure;
}
   
sub uvspec_run_test {
    my ($inp, $out, $exinp, $exout, $test) = @_;
    my $diffout= "TMP_DIFF.OUT";
    system("cat  $exinp > $inp");
    system("$UVTools::uvspec < $inp > $out");
    if ( $answers == 1 ) {
	system("mv $out $exout");
	system("rm $inp");
    }
    else {
	system("perl ../tools/ndiff --limit 0.01 --maxdiff 0.005 $exout $out > $diffout");
	open(FILE,$diffout);
	$ndiffs = <FILE>;
	close(FILE);
	chop $ndiffs;
	printf stderr "Found %d serious differences for $test test.\n";
	system("rm $diffout $inp $out");
	return $ndiffs;
    }
}

# Local Variables:
# mode: Perl
# End:
